#include <Wire.h>
#include "RTClib.h"
#include "OneWire.h"
#include <EEPROM.h>
#include "Nixies.h"
#include "Registers.h"

#include <Adafruit_NeoPixel.h>

OneWire ds(4); // Temperature Sensor on pin 4
RTC_DS1307 RTC;

const int led1 =  9;      // the number of the LED pin
const int led2 =  10;      // the number of the LED pin

#define modeBtn   8
#define hoursBtn   14
#define minutesBtn   15
#define lightBtn   16
#define speaker   3
#define hvPower   2
#define pixel   A3

#define PIXEL_PIN    A3    // Digital IO pin connected to the NeoPixels.
#define PIXEL_COUNT 6
// Parameter 1 = number of pixels in strip,  neopixel stick has 8
// Parameter 2 = pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed:
//   NEO_RGB     Pixels are wired for RGB bitstream
//   NEO_GRB     Pixels are wired for GRB bitstream, correct for neopixel stick
//   NEO_KHZ400  400 KHz bitstream (e.g. FLORA pixels)
//   NEO_KHZ800  800 KHz bitstream (e.g. High Density LED strip), correct for neopixel stick
Adafruit_NeoPixel strip = Adafruit_NeoPixel(PIXEL_COUNT, PIXEL_PIN, NEO_GRB + NEO_KHZ400);

unsigned int serialPin = 7 ;
unsigned int serialClockPin = 6;
unsigned int registerClockPin = 5;

int hour, minute, second; 
int light;
byte rainbowX;
static unsigned long lastTick = 0; // set up a local variable to hold the last time we moved forward one second

Nixies nixies;

void setup() {
//	Serial.begin(9600);
	Wire.begin();
	
	RTC.begin();
	if (!RTC.isrunning()) {
//		Serial.println("RTC needs to be set");
		RTC.adjust(DateTime(__DATE__, __TIME__));
	}

 pinMode(led1, OUTPUT);
 pinMode(led2, OUTPUT);   
 pinMode(speaker, OUTPUT);   
 pinMode(hvPower, OUTPUT);  
 digitalWrite(hvPower,HIGH); //power on high voltage power supply
 pinMode(pixel, OUTPUT);    
  
 pinMode(modeBtn, INPUT_PULLUP);
 pinMode(hoursBtn, INPUT_PULLUP);    
 pinMode(minutesBtn, INPUT_PULLUP);
 pinMode(lightBtn, INPUT_PULLUP);

	nixies.initialize(serialPin, serialClockPin, registerClockPin, 6);
 //   RTC.adjust(DateTime(2014, 1, 21, 0, 0, 0));
      strip.begin();
  strip.show(); // Initialize all pixels to 'off'

  light = EEPROM.read(10);
  rainbowX=0;
  startShow( light);
}

void displayTime() {

//	nixies.setNixie(5, hour/10);
//	nixies.setNixie(4, hour%10);
//	nixies.setNixie(3, minute/10);
//	nixies.setNixie(2, minute%10);
//	nixies.setNixie(1, second/10);
//	nixies.setNixie(0, second%10);
//	nixies.update();

  nixies.setNixie(4, hour/10);
  nixies.setNixie(5, hour%10);
  nixies.setNixie(2, minute/10);
  nixies.setNixie(3, minute%10);
  nixies.setNixie(0, second/10);
  nixies.setNixie(1, second%10);
  nixies.update();

}

void loop() {
	  DateTime now = RTC.now();
   hour = now.hour();
   minute = now.minute();
   second = now.second();



if ((minute==0) && (second==0)){
  beep();
  displayTime();
 rainbowCycle(10);
 displayTemperature();
 startShow(light);
// testTubes();
}

if (second==0)
{
  testTubes();
}


//////////////////////////////////////////////////////////////////  


  if (digitalRead(hoursBtn)==LOW)
  {
    hour++;
    if (hour > 23) {
      hour = 0;
    }
    if (hour < 0) {
      hour = 23;
    }
  
    digitalWrite(led2, HIGH);
    delay(110);
    digitalWrite(led2, LOW);
    delay(110); 
     second=1; 
    RTC.adjust(DateTime(2014, 1, 21, hour, minute, second));
      playTone( 100, 8000);
  }

  if (digitalRead(minutesBtn)==LOW)
  {
     minute++;
    if (minute > 59) {
      minute = 0;
    }
    if (minute < 0) {
      minute = 59;
    }

    digitalWrite(led2, HIGH);
    delay(110);
    digitalWrite(led2, LOW);
    delay(110); 
     second=1; 
    RTC.adjust(DateTime(2014, 1, 21, hour, minute, second));
     playTone( 100, 8000);
  }

    if (digitalRead(modeBtn)==LOW)
  {
    playTone( 100, 11000); 
    displayTemperature();
//    testTubes();
  }

    if (digitalRead(lightBtn)==LOW)
  {
   beep();
           light++;
      if (light > 4)
        light=0;

      startShow(light);
      EEPROM.write(10, light);
  }

  displayTime();

  if ((millis() - lastTick) >= 100) {
     rainbowX++;
    lastTick = millis();
  } 

      
      if (light == 4)
      {
  uint16_t i, j;
 //   rainbowX++;
    for(i=0; i< strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel(((i * 256 / strip.numPixels()) + rainbowX) & 255));
    }
    strip.show();

        
      }
}

void beep ()
{
     playTone( 50, 6000);
     playTone( 50, 5000);
     playTone( 50, 8000);
}

void startShow(int i) {
  switch(i){
    case 0: colorWipe(strip.Color(0, 0, 0), 50);    // Black/off
            break;
    case 1: colorWipe(strip.Color(255, 0, 0), 250);  // Red
            break;
    case 2: colorWipe(strip.Color(0, 255, 0), 250);  // Green
            break;
    case 3: colorWipe(strip.Color(0, 0, 255), 250);  // Blue
            break;
    case 4: rainbowCycle(1);
            rainbowCycle(3);
            break;

            
  }
}


// Fill the dots one after the other with a color
void colorWipe(uint32_t c, uint8_t wait) {
  for(uint16_t i=0; i<strip.numPixels(); i++) {
    strip.setPixelColor(i, c);
    strip.show();
    delay(wait);
  }
}

void rainbow(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256; j++) {
    for(i=0; i<strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel((i+j) & 255));
    }
    strip.show();
    delay(wait);
  }
}

// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycle(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256; j++) { // 5 cycles of all colors on wheel
    for(i=0; i< strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel(((i * 256 / strip.numPixels()) + j) & 255));
    }
    strip.show();
    delay(wait);
  }
}
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if(WheelPos < 85) {
    return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  }
  if(WheelPos < 170) {
    WheelPos -= 85;
    return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
  WheelPos -= 170;
  return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}




// Code for this function from http://bildr.org/2011/07/ds18b20-arduino/
// Copyright (c) 2010 bildr community

float getTemperature() {
	//returns the temperature from one DS18S20 in DEG Celsius
	byte data[12];
	byte addr[8];
	
	if (!ds.search(addr)) {
		//no more sensors on chain, reset search
		ds.reset_search();
		return -1000;
	}
	
	if (OneWire::crc8(addr, 7)!=addr[7]) {
//		Serial.println("CRC is not valid!");
		return -1000;
	}

	if (addr[0]!=0x10 && addr[0]!=0x28) {
//		Serial.print("Device is not recognized");
		return -1000;
	}

	ds.reset();
	ds.select(addr);
	ds.write(0x44,1); // start conversion, with parasite power on at the end

	byte present = ds.reset();
	ds.select(addr);
	ds.write(0xBE); // Read Scratchpad


	for (int i=0; i<9; i++) { // we need 9 bytes
		data[i] = ds.read();
	}

	ds.reset_search();

int HighByte, LowByte, TReading, SignBit, Tc_100, Whole, Fract;

  LowByte = data[0];
  HighByte = data[1];
  TReading = (HighByte << 8) + LowByte;
  SignBit = TReading & 0x8000;  // test most sig bit
  if (SignBit) // negative
  {
    TReading = (TReading ^ 0xffff) + 1; // 2's comp
  }
  Tc_100 = (6 * TReading) + TReading / 4;    // multiply by (100 * 0.0625) or 6.25
  Whole = Tc_100 / 100;  // separate off the whole and fractional portions

//	float tempRead = ((MSB << 8) | LSB); //using two's compliment
//	float TemperatureSum = tempRead / 16;

	return Whole;
}


void displayTemperature() {
  int temperature = getTemperature();
colorWipe(strip.Color(0, 0, 0), 50);    // Black/off  
  nixies.setNixie(4, temperature/10);
  nixies.setNixie(5, temperature%10);
  nixies.setNixie(3, 10);
  nixies.setNixie(2, 10);
  nixies.setNixie(1, 10);
  nixies.setNixie(0, 10);
  nixies.update();
  // fade in from min to max in increments of 5 points:
  for (int fadeValue = 0 ; fadeValue <= 255; fadeValue += 5) {
    // sets the value (range from 0 to 255):
    analogWrite(led1, fadeValue);
    // wait for 30 milliseconds to see the dimming effect
    delay(20);

  }

  // fade out from max to min in increments of 5 points:
  for (int fadeValue = 255 ; fadeValue >= 0; fadeValue -= 5) {
    // sets the value (range from 0 to 255):
    analogWrite(led1, fadeValue);
    // wait for 30 milliseconds to see the dimming effect
    delay(30);
  }
 startShow(light);
}

void testTubes() {
  // Test every digit
  for (int i=0; i<10; i++) {
   DateTime now = RTC.now();
   second = now.second();
    
  nixies.setNixie(5, i);
  nixies.setNixie(4, i);
  nixies.setNixie(3, i);
  nixies.setNixie(2, i);
  nixies.setNixie(1, second/10);
  nixies.setNixie(0, second%10);
    nixies.update();
    digitalWrite(led2, HIGH);
    delay(1);
    digitalWrite(led2, LOW);
    delay(200);
  }

//  // Test setting to blank
//  nixies.setNixie(0, 10);
//  nixies.setNixie(1, 10);
//  nixies.setNixie(2, 10);
//  nixies.setNixie(3, 10);
//  nixies.setNixie(4, 10);
//  nixies.setNixie(5, 10);
//  nixies.update();
//
//    digitalWrite(led1, HIGH);
//    delay(300);
//    digitalWrite(led1, LOW);
}

void playTone(long duration, int freq) {
// duration in mSecs, frequency in hertz

  duration *= 700;
  int period = (1.0 / freq) * 1000000;
  long elapsed_time = 0;
  int amax=0;
  byte up=1;

    while (elapsed_time < duration) {
      digitalWrite(speaker,HIGH);
      delayMicroseconds(period / 2);
      digitalWrite(speaker, LOW);
      delayMicroseconds(period / 2);
      elapsed_time += (period);  
  }  
}

